#include<stdio.h>
#include<CUnit/Basic.h>
#include<string.h>
#include"EnsembleDEntiers.h"

#define TRUE 1
#define FALSE 0

int init_suite_success(void) { 
  return 0; 
}

int clean_suite_success(void) { 
  return 0; 
}

EDE_Ensemble creer_ensemble_123() {
  EDE_Ensemble e=EDE_ensemble();
  EDE_ajouter(&e,1);
  EDE_ajouter(&e,2);
  EDE_ajouter(&e,3);  
  return e;
}

EDE_Ensemble creer_ensemble_145() {
  EDE_Ensemble e=EDE_ensemble();
  EDE_ajouter(&e,1);
  EDE_ajouter(&e,4);
  EDE_ajouter(&e,5);  
  return e;
}

void test_ensemble_vide(void){
  EDE_Ensemble e=EDE_ensemble();

  CU_ASSERT_TRUE(EDE_cardinalite(e)==0);

  EDE_vider(&e);
}

void test_ajouter_non_present(void){
  int c1,c2;
  EDE_Ensemble e=creer_ensemble_123();
  c1=EDE_cardinalite(e);
  EDE_ajouter(&e,4);
  c2=EDE_cardinalite(e);  

  CU_ASSERT_TRUE(c1+1==c2);

  EDE_vider(&e);
}

void test_ajouter_present(void){
  int c1,c2;
  EDE_Ensemble e=creer_ensemble_123();
  c1=EDE_cardinalite(e);
  EDE_ajouter(&e,1);
  c2=EDE_cardinalite(e); 

  CU_ASSERT_TRUE(c1==c2);

  EDE_vider(&e);
}

void test_present_apres_ajout(void){
  EDE_Ensemble e=creer_ensemble_123();
  EDE_ajouter(&e,5); 

  CU_ASSERT_TRUE(EDE_estPresent(e,5));

  EDE_vider(&e);
}


void test_retirer_present(void){
  int c1,c2;
  EDE_Ensemble e=creer_ensemble_123();
  c1=EDE_cardinalite(e);
  EDE_retirer(&e,2);
  c2=EDE_cardinalite(e);  

  CU_ASSERT_TRUE(c1-1==c2);

  EDE_vider(&e);
}

void test_retirer_non_present(void){
  int c1,c2;
  EDE_Ensemble e=creer_ensemble_123();
  c1=EDE_cardinalite(e);
  EDE_retirer(&e,4);
  c2=EDE_cardinalite(e); 

  CU_ASSERT_TRUE(c1==c2);

  EDE_vider(&e);
}

void test_absent_apres_retrait(void){
  EDE_Ensemble e=creer_ensemble_123();
  EDE_retirer(&e,2); 
  CU_ASSERT_FALSE(EDE_estPresent(e,2));
  EDE_vider(&e);
}

void test_union(void){
  EDE_Ensemble e1=creer_ensemble_123();
  EDE_Ensemble e2=creer_ensemble_145();
  EDE_Ensemble e3=EDE_union(e1,e2);

  CU_ASSERT_TRUE(EDE_estPresent(e3,1) && EDE_estPresent(e3,2) && EDE_estPresent(e3,3) && EDE_estPresent(e3,4) && EDE_estPresent(e3,1) && EDE_estPresent(e3,5));

  EDE_vider(&e1);
  EDE_vider(&e2);
  EDE_vider(&e3);
}

void test_intersection(void){
  EDE_Ensemble e1=creer_ensemble_123();
  EDE_Ensemble e2=creer_ensemble_145();
  EDE_Ensemble e3=EDE_intersection(e1,e2);

  CU_ASSERT_TRUE(EDE_estPresent(e3,1) && !EDE_estPresent(e3,2) && !EDE_estPresent(e3,3) && !EDE_estPresent(e3,4)  && !EDE_estPresent(e3,5));

  EDE_vider(&e1);
  EDE_vider(&e2);
  EDE_vider(&e3);
}

void test_soustraction(void){
  EDE_Ensemble e1=creer_ensemble_123();
  EDE_Ensemble e2=creer_ensemble_145();
  EDE_Ensemble e3=EDE_soustraction(e1,e2);

  CU_ASSERT_TRUE(!EDE_estPresent(e3,1) && EDE_estPresent(e3,2) && EDE_estPresent(e3,3));

  EDE_vider(&e1);
  EDE_vider(&e2);
  EDE_vider(&e3);
}

void test_egalite_meme_ensemble(void){
  EDE_Ensemble e1=creer_ensemble_123();

  CU_ASSERT_TRUE(EDE_egale(e1,e1));

  EDE_vider(&e1);
}

void test_egalite_ensembles_differents(void){
  EDE_Ensemble e1=creer_ensemble_123();
  EDE_Ensemble e2=creer_ensemble_145();

  CU_ASSERT_FALSE(EDE_egale(e1,e2));

  EDE_vider(&e1);
  EDE_vider(&e2);
}

void test_copier(void){
  EDE_Ensemble e1=creer_ensemble_123();
  EDE_Ensemble e2=EDE_copier(e1);

  CU_ASSERT_TRUE(EDE_egale(e1,e2));

  EDE_vider(&e1);
  EDE_vider(&e2);
}


int main(int argc, char** argv){
  CU_pSuite pSuite = NULL;

  /* initialisation du registre de tests */
  if (CUE_SUCCESS != CU_initialize_registry())
    return CU_get_error();

  /* ajout d'une suite de test */
  pSuite = CU_add_suite("Tests boite noire", init_suite_success, clean_suite_success);
  if (NULL == pSuite) {
    CU_cleanup_registry();
    return CU_get_error();
  }

  /* Ajout des tests  la suite de tests boite noire */
  if ((NULL == CU_add_test(pSuite, "1 - la creation d'un ensemble doit etre vide", test_ensemble_vide)) 
      || (NULL == CU_add_test(pSuite, "2 - ajouter un element non present incremente la cardinalite", test_ajouter_non_present))  
      || (NULL == CU_add_test(pSuite, "3 - ajouter un element present n'incremente pas la cardinalite", test_ajouter_present)) 
      || (NULL == CU_add_test(pSuite, "4 - un element ajoute est present", test_present_apres_ajout)) 
      || (NULL == CU_add_test(pSuite, "5 - retirer un element non present decremente la cardinalite", test_retirer_non_present))  
      || (NULL == CU_add_test(pSuite, "6 - retirer un element present ne decremente pas la cardinalite", test_retirer_present)) 
      || (NULL == CU_add_test(pSuite, "7 - un element retire n'est plus present", test_absent_apres_retrait)) 
      || (NULL == CU_add_test(pSuite, "8 - union", test_union)) 
      || (NULL == CU_add_test(pSuite, "9 - intersection", test_intersection)) 
      || (NULL == CU_add_test(pSuite, "10 - soustraction", test_intersection)) 
      || (NULL == CU_add_test(pSuite, "11 - un ensemble est egal a lui meme", test_egalite_meme_ensemble))
      || (NULL == CU_add_test(pSuite, "12 - un ensemble est different d'un autre ensemble", test_egalite_ensembles_differents))
      || (NULL == CU_add_test(pSuite, "13 - un ensemble est egal a une de ses copies", test_copier)) 
      )
    {
      CU_cleanup_registry();
      return CU_get_error();
    }

  /* Lancement des tests */
  CU_basic_set_mode(CU_BRM_VERBOSE);
  CU_basic_run_tests();
  printf("\n");
  CU_basic_show_failures(CU_get_failure_list());
  printf("\n\n");

  /* Nettoyage du registre */
  CU_cleanup_registry();
  return CU_get_error();
}
